var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import React, { useMemo } from 'react';
import PropTypes from 'prop-types';
import flatten from 'lodash/flatten';
import { cx, css } from '@leafygreen-ui/emotion';
import { fontFamilies } from '@leafygreen-ui/tokens';
// Import from core so we can register the appropriate languages ourselves
import hljs from 'highlight.js/lib/core';
import hljsDefineGraphQL from 'highlightjs-graphql';
import { Language, Mode } from './types';
import { SupportedLanguages, languageParsers } from './languages';
import { injectGlobalStyles } from './globalStyles';
import renderingPlugin, { TableContent } from './renderingPlugin';
import { SyntaxContext } from './SyntaxContext';
export function expandRangeTuple(tuple) {
    var _a = __spreadArrays(tuple).map(function (bound) { return Math.max(bound, 0); }).sort(), lower = _a[0], upper = _a[1];
    var maxHighlightingDifference = 499;
    // Make sure passing large numbers doesn't freeze the browser
    var clampedUpperBound = Math.min(lower + maxHighlightingDifference, upper);
    var expandedRange = [];
    for (var i = lower; i <= clampedUpperBound; i++) {
        expandedRange.push(i);
    }
    return expandedRange;
}
export function normalizeLineHighlightingDefinition(numbers) {
    return flatten(numbers.map(function (item) {
        if (item instanceof Array) {
            return expandRangeTuple(item);
        }
        return item;
    }));
}
function filterSupportedLanguages(language) {
    return language !== 'cs' && language !== 'js' && language !== 'ts';
}
var syntaxHighlightingInitialized = false;
function initializeSyntaxHighlighting() {
    syntaxHighlightingInitialized = true;
    injectGlobalStyles();
    // We filter out 'cs' here because it's redundant with 'csharp' and 'js' because it's redundant with 'javascript'
    var SupportedLanguagesList = Object.values(SupportedLanguages).filter(filterSupportedLanguages);
    SupportedLanguagesList.forEach(function (language) {
        if (language === 'graphql') {
            hljsDefineGraphQL(hljs);
        }
        else {
            hljs.registerLanguage(language, languageParsers[language]);
        }
    });
    hljs.configure({
        languages: SupportedLanguagesList,
        classPrefix: 'lg-highlight-',
        tabReplace: '  ',
    });
    hljs.addPlugin(renderingPlugin);
}
var codeStyles = css(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  color: inherit;\n  font-size: 13px;\n  font-family: ", ";\n  line-height: 24px;\n"], ["\n  color: inherit;\n  font-size: 13px;\n  font-family: ", ";\n  line-height: 24px;\n"])), fontFamilies.code);
function Syntax(_a) {
    var children = _a.children, language = _a.language, _b = _a.darkMode, darkMode = _b === void 0 ? false : _b, _c = _a.showLineNumbers, showLineNumbers = _c === void 0 ? false : _c, _d = _a.highlightLines, highlightLines = _d === void 0 ? [] : _d, className = _a.className, rest = __rest(_a, ["children", "language", "darkMode", "showLineNumbers", "highlightLines", "className"]);
    if (!syntaxHighlightingInitialized) {
        initializeSyntaxHighlighting();
    }
    var highlightedContent = useMemo(function () {
        if (language === Language.None) {
            return null;
        }
        return hljs.highlight(language, children, true);
    }, [language, children]);
    var content = highlightedContent === null ? (
    // We create a similar data structure to the rendering plugin so that we can generate
    // a table that's identical when the plugin isn't being used.
    React.createElement(TableContent, { lines: children.split('\n').map(function (item) { return (item ? [item] : []); }) })) : (highlightedContent.react);
    var mode = darkMode ? Mode.Dark : Mode.Light;
    var parsedHighlightLines = normalizeLineHighlightingDefinition(highlightLines);
    return (React.createElement(SyntaxContext.Provider, { value: {
            highlightLines: parsedHighlightLines,
            showLineNumbers: showLineNumbers,
            darkMode: darkMode,
        } },
        React.createElement("code", __assign({}, rest, { className: cx("lg-highlight-hljs-" + mode, codeStyles, language, className) }),
            React.createElement("table", { className: css(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n            border-spacing: 0;\n          "], ["\n            border-spacing: 0;\n          "]))) },
                React.createElement("tbody", null, content)))));
}
Syntax.displayName = 'Syntax';
Syntax.propTypes = {
    children: PropTypes.string.isRequired,
    language: PropTypes.oneOf(Object.values(Language)),
    className: PropTypes.string,
    darkMode: PropTypes.bool,
    showLineNumbers: PropTypes.bool,
    highlightLines: PropTypes.arrayOf(PropTypes.oneOfType([
        PropTypes.arrayOf(PropTypes.number),
        PropTypes.number,
    ])),
};
export default Syntax;
var templateObject_1, templateObject_2;
//# sourceMappingURL=Syntax.js.map