var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
import React from 'react';
import { css, cx } from '@leafygreen-ui/emotion';
import { uiColors } from '@leafygreen-ui/palette';
import { useSyntaxContext } from './SyntaxContext';
function Token(_a) {
    var kind = _a.kind, children = _a.children;
    var className = kind ? "lg-highlight-" + kind : '';
    return React.createElement("span", { className: className }, children);
}
function isArray(item) {
    return item != null && item instanceof Array;
}
function isObject(item) {
    return item != null && typeof item === 'object' && !(item instanceof Array);
}
function isString(item) {
    return item != null && typeof item === 'string';
}
export function processToken(token, key) {
    if (token == null) {
        return null;
    }
    if (isString(token)) {
        return token;
    }
    if (isArray(token)) {
        return token.map(processToken);
    }
    if (isObject(token)) {
        return (React.createElement(Token, { key: key, kind: token.kind }, processToken(token.children)));
    }
    return token;
}
var cellStyle = css(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n  border-spacing: 0;\n  padding: 0;\n  vertical-align: top;\n\n  &:first-of-type {\n    padding-left: 12px;\n  }\n\n  &:last-of-type {\n    padding-right: 12px;\n  }\n"], ["\n  border-spacing: 0;\n  padding: 0;\n  vertical-align: top;\n\n  &:first-of-type {\n    padding-left: 12px;\n  }\n\n  &:last-of-type {\n    padding-right: 12px;\n  }\n"])));
function getHighlightedRowStyle(darkMode) {
    var backgroundColor, backgroundImage, borderColor;
    if (darkMode) {
        backgroundColor = 'transparent';
        backgroundImage = "linear-gradient(90deg, " + uiColors.gray.dark3 + ", transparent)";
        borderColor = uiColors.gray.dark3;
    }
    else {
        backgroundColor = uiColors.yellow.light3;
        backgroundImage = 'none';
        borderColor = uiColors.yellow.light2;
    }
    return css(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n    background-color: ", ";\n    background-image: ", ";\n\n    // Selects all children of a highlighted row, and adds a border top\n    & > td {\n      border-top: 1px solid ", ";\n    }\n\n    // Selects following rows after a highlighted row, and adds a border top\n    // We don't add border bottoms here to support consecutive highlighted rows.\n    & + tr > td {\n      border-top: 1px solid ", ";\n    }\n\n    // Remove borders between consecutive highlighted rows\n    & + & > td {\n      border-top: 0;\n    }\n\n    // If the highlighted row is the last child, then we add a border bottom\n    &:last-child > td {\n      border-bottom: 1px solid ", ";\n    }\n  "], ["\n    background-color: ", ";\n    background-image: ", ";\n\n    // Selects all children of a highlighted row, and adds a border top\n    & > td {\n      border-top: 1px solid ", ";\n    }\n\n    // Selects following rows after a highlighted row, and adds a border top\n    // We don't add border bottoms here to support consecutive highlighted rows.\n    & + tr > td {\n      border-top: 1px solid ", ";\n    }\n\n    // Remove borders between consecutive highlighted rows\n    & + & > td {\n      border-top: 0;\n    }\n\n    // If the highlighted row is the last child, then we add a border bottom\n    &:last-child > td {\n      border-bottom: 1px solid ", ";\n    }\n  "])), backgroundColor, backgroundImage, borderColor, borderColor, borderColor);
}
export function LineTableRow(_a) {
    var _b;
    var lineNumber = _a.lineNumber, highlighted = _a.highlighted, darkMode = _a.darkMode, children = _a.children;
    var numberColor = uiColors.gray[darkMode ? 'dark1' : 'light1'];
    var highlightedNumberColor = darkMode
        ? uiColors.gray.light2
        : uiColors.yellow.dark2;
    return (React.createElement("tr", { className: cx((_b = {}, _b[getHighlightedRowStyle(darkMode)] = highlighted, _b)) },
        lineNumber && (React.createElement("td", { className: cx(cellStyle, css(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n              padding-right: 24px;\n              user-select: none;\n              color: ", ";\n            "], ["\n              padding-right: 24px;\n              user-select: none;\n              color: ", ";\n            "])), highlighted ? highlightedNumberColor : numberColor)) }, lineNumber)),
        React.createElement("td", { className: cellStyle }, children)));
}
export function treeToLines(children) {
    var lines = [];
    var currentLineIndex = 0;
    // Create a new line, if no lines exist yet
    if (lines[currentLineIndex] == null) {
        lines[currentLineIndex] = [];
    }
    children.forEach(function (child) {
        if (isString(child)) {
            // If the current element is a string that includes a line break, we need to handle it differently
            if (child.includes('\n')) {
                child.split('').forEach(function (fragment) {
                    if (fragment === '\n') {
                        // If the fragment is a new line character, we create a new line
                        currentLineIndex++;
                        lines[currentLineIndex] = [];
                    }
                    else {
                        var currentIndexInLine = lines[currentLineIndex].length - 1;
                        if (isString(lines[currentLineIndex][currentIndexInLine])) {
                            // If the last element in the line is a string, we append this string to it
                            lines[currentLineIndex][currentIndexInLine] += fragment;
                        }
                        else {
                            // Otherwise, we push the string fragment on its own
                            lines[currentLineIndex].push(fragment);
                        }
                    }
                });
            }
            else {
                // We don't need to do anything special in the case where the string doesn't contain a line break
                lines[currentLineIndex].push(child);
            }
        }
        // Line breaks aren't a part of token objects, so we can assume those objects go on the current line
        if (isObject(child)) {
            lines[currentLineIndex].push(child);
        }
    });
    return lines;
}
export function TableContent(_a) {
    var _b, _c;
    var lines = _a.lines;
    var _d = useSyntaxContext(), highlightLines = _d.highlightLines, showLineNumbers = _d.showLineNumbers, darkMode = _d.darkMode;
    var trimmedLines = __spreadArrays(lines);
    // Strip empty lines from the beginning of code blocks
    while (((_b = trimmedLines[0]) === null || _b === void 0 ? void 0 : _b.length) === 0) {
        trimmedLines.shift();
    }
    // Strip empty lines from the end of code blocks
    while (((_c = trimmedLines[trimmedLines.length - 1]) === null || _c === void 0 ? void 0 : _c.length) === 0) {
        trimmedLines.pop();
    }
    return (React.createElement(React.Fragment, null, trimmedLines.map(function (line, index) {
        var currentLineNumber = index + 1;
        var highlightLine = highlightLines.includes(currentLineNumber);
        var displayLineNumber;
        if (showLineNumbers) {
            displayLineNumber = currentLineNumber;
        }
        var processedLine = (line === null || line === void 0 ? void 0 : line.length) ? (line.map(processToken)) : (
        // We create placeholder content when a line break appears to preserve the line break's height
        // It needs to be inline-block for the table row to not collapse.
        React.createElement("div", { className: css(templateObject_4 || (templateObject_4 = __makeTemplateObject(["\n              display: inline-block;\n            "], ["\n              display: inline-block;\n            "]))) }));
        return (React.createElement(LineTableRow, { key: currentLineNumber, lineNumber: displayLineNumber, darkMode: darkMode, highlighted: highlightLine }, processedLine));
    })));
}
var plugin = {
    'after:highlight': function (result) {
        var rootNode = result.emitter.rootNode;
        result.react = React.createElement(TableContent, { lines: treeToLines(rootNode.children) });
    },
};
export default plugin;
var templateObject_1, templateObject_2, templateObject_3, templateObject_4;
//# sourceMappingURL=renderingPlugin.js.map